
"""
This module 'populate' defines methods to populate the database with predefined ('static') data,
according to the proposed Django way: https://docs.djangoproject.com/en/2.0/topics/migrations/#data-migrations

import this module in your empty migration step file, and add the following migration:

from ..populate import *

class Migration(migrations.Migration):

    dependencies = [ <the dependency is automatically inserted here> ]

    operations = [ migrations.RunPython(populate_choices) ]

"""

from .models.specification import RoleChoice, DatatypeChoice, DataformatChoice, CopyReasonChoice
from .models.scheduling import TaskStateChoice, TaskTypeChoice, StationTypeChoice

def populate_choices(apps, schema_editor):
    '''
    populate each 'choice' table in the database with the 'static' list of 'choice'.Choices for
    each 'choice'type in RoleChoice, DatatypeChoice, DataformatChoice, CopyReasonChoice
    :return: None
    '''
    for choice_class in [RoleChoice, DatatypeChoice, DataformatChoice, CopyReasonChoice,
                         TaskStateChoice, TaskTypeChoice, StationTypeChoice]:
        choice_class.objects.bulk_create([choice_class(value=x.value) for x in choice_class.Choices])

