"""
This file contains the serializers (for the elsewhere defined data models)
"""

from rest_framework import serializers
from .. import models
from rest_framework import decorators


class TagsSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.Tags
        fields = '__all__'


class GeneratorTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.GeneratorTemplate
        fields = '__all__'


class DefaultGeneratorTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DefaultGeneratorTemplate
        fields = '__all__'


class RunTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.RunTemplate
        fields = '__all__'


class DefaultRunTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DefaultRunTemplate
        fields = '__all__'


class WorkRequestTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.WorkRequestTemplate
        fields = '__all__'


class DefaultWorkRequestTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DefaultWorkRequestTemplate
        fields = '__all__'


class WorkRelationSelectionTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.WorkRelationSelectionTemplate
        fields = '__all__'


class DefaultWorkRelationSelectionTemplateSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.DefaultWorkRelationSelectionTemplate
        fields = '__all__'


class RoleChoiceSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.RoleChoice
        fields = '__all__'


class DatatypeChoiceSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.DatatypeChoice
        fields = '__all__'


class DataformatChoiceSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.DataformatChoice
        fields = '__all__'


class CopyReasonChoiceSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.CopyReasonChoice
        fields = '__all__'


class WorkIORoleSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.WorkIORole
        fields = '__all__'


class CycleSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.Cycle
        fields = '__all__'


class ProjectSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.Project
        fields = '__all__'


class RunSetSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.RunSet
        fields = '__all__'


class RunDraftSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.RunDraft
        fields = '__all__'


class RunBlueprintSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.RunBlueprint
        fields = '__all__'


class WorkRequestDraftSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.WorkRequestDraft
        fields = '__all__'


class WorkRequestBlueprintSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.WorkRequestBlueprint
        fields = '__all__'


class WorkRequestRelationDraftSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.WorkRequestRelationDraft
        fields = '__all__'


class WorkRequestRelationBlueprintSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.WorkRequestRelationBlueprint
        fields = '__all__'


# ----- JSON


class WorkRequestBlueprintSerializerReactJSONform(serializers.HyperlinkedModelSerializer):

    # Create a JSON editor form to replace the simple text field based on the schema in the template that this
    # blueprint refers to. If that fails, the JSONField remains a standard text input.
    #
    # Note: I feel a bit uneasy with this since I feel there should be a more straight-forward solution tham
    #  ...intercepting the init process to determine the schema (or template uri or so) for the style attribute.
    #  ...Hoewever, I did not manage to simply pass the value(!) of e.g. the template field as a style attribute
    #  ...of the JSONField via a SerializerMethodField or similar, although I feel that should be possible.
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        import json

        try:
            schema = self.instance.template.schema

            self.fields['requirements_doc'] = serializers.JSONField(
                style={'template': 'react_jsonschema_form_widget.html',
                       'schema': json.dumps(schema)})

        except:
            # todo: Shall we use one of the default templates for the init?
            print('Could not determine schema, hence no fancy JSON form. Expected for list view.')


    class Meta:
        model = models.WorkRequestBlueprint
        fields = '__all__'


class WorkRequestBlueprintSerializerJSONeditor(serializers.HyperlinkedModelSerializer):

    # Create a JSON editor form to replace the simple text field based on the schema in the template that this
    # blueprint refers to. If that fails, the JSONField remains a standard text input.
    #
    # Note: I feel a bit uneasy with this since I feel there should be a more straight-forward solution tham
    #  ...intercepting the init process to determine the schema (or template uri or so) for the style attribute.
    #  ...Hoewever, I did not manage to simply pass the value(!) of e.g. the template field as a style attribute
    #  ...of the JSONField via a SerializerMethodField or similar, although I feel that should be possible.
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        import json

        try:
            schema = self.instance.template.schema
            self.fields['requirements_doc'] = serializers.JSONField(
                style={'template': 'json_editor_widget.html',
                       'schema': json.dumps(schema)})

        except:
            # todo: Shall we use one of the default templates for the init?
            print('Could not determine schema, hence no fancy JSON form. Expected for list view.')


    #requirements_doc = serializers.JSONField(style={'template': 'json_editor_form_2.html', 'template_schema_uri': "http://localhost:8000/work_request_template/2/?format=json"})
    class Meta:
        model = models.WorkRequestBlueprint
        fields = '__all__'

class WorkRequestBlueprintSerializerJSONeditorOnline(serializers.HyperlinkedModelSerializer):

    # Create a JSON editor form to replace the simple text field based on the schema in the template that this
    # blueprint refers to. If that fails, the JSONField remains a standard text input.
    #
    # Note: I feel a bit uneasy with this since I feel there should be a more straight-forward solution tham
    #  ...intercepting the init process to determine the schema (or template uri or so) for the style attribute.
    #  ...Hoewever, I did not manage to simply pass the value(!) of e.g. the template field as a style attribute
    #  ...of the JSONField via a SerializerMethodField or similar, although I feel that should be possible.
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        import json

        try:
            schema = self.instance.template.schema
            self.fields['requirements_doc'] = serializers.JSONField(
                style={'template': 'josdejong_jsoneditor_widget.html',
                       'schema': json.dumps(schema)})

        except:
            # todo: Shall we use one of the default templates for the init?
            print('Could not determine schema, hence no fancy JSON form. Expected for list view.')


    #requirements_doc = serializers.JSONField(style={'template': 'json_editor_form_2.html', 'template_schema_uri': "http://localhost:8000/work_request_template/2/?format=json"})
    class Meta:
        model = models.WorkRequestBlueprint
        fields = '__all__'



class WorkRequestBlueprintJSONSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.WorkRequestBlueprint
        fields = ('requirements_doc',)


